/*********************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

#include "p30fxxxx.h"
#include "G711Lib_common.h"
#include "G711Lib_Si3000.h"
#include "g711.h"
#include <dsp.h>

_FOSC(CSW_FSCM_OFF & ECIO_PLL8);   // EC with 8xPLL, Failsafe clock off
_FWDT(WDT_OFF);                  // Watchdog timer disabled
_FBORPOR(PBOR_OFF & MCLR_EN);    // Brown-out reset disabled, MCLR reset enabled
//_FGS(CODE_PROT_OFF);             // Code protect disabled

/*-----------------------------------------------------------------------------
                                  Global Variables
------------------------------------------------------------------------------*/
// User defined ping pong
short buffer1[NUMOFSAMPLESPERBLOCK];        //sampleIpBuffer.
short buffer2[NUMOFSAMPLESPERBLOCK];    	//sampleOpBuffer.
short expandbuffer1[NUMOFSAMPLESPERBLOCK];	//sampleExpIpBuffer
short expandbuffer2[NUMOFSAMPLESPERBLOCK];	//sampleExpOpBuffer
short buffer11[NUMOFSAMPLESPERBLOCK];   	//sampleComprsIpBuffer.
short buffer12[NUMOFSAMPLESPERBLOCK];		//sampleComprsOpBuffer.
long slen = NUMOFSAMPLESPERBLOCK;
short  *temp, *temp1;

void INTx_IO_Init(void);
extern volatile int UseFilter;
/*-----------------------------------------------------------------------------
                                  Macro instantiation
------------------------------------------------------------------------------*/
G711SI3000INIT                     //Macro to instantiate SI3000 structure
CODECSETUP                         //Macro to instantiate CODECSETUP structure

/*-----------------------------------------------------------------------------
                                  Filter Setup
------------------------------------------------------------------------------*/
#define FILTER_BLOCK_LENGTH 256

extern FIRStruct LowPassFilter;

fractional SigIn[FILTER_BLOCK_LENGTH] __attribute__ ((space(ymemory), near, aligned (FILTER_BLOCK_LENGTH * 2)));
fractional FiltOut[FILTER_BLOCK_LENGTH] __attribute__ (( space(xmemory) ));

/******************************************************************************
**
**  FunctionName: G711SI3000LoopBack_Main ( )
**
**  Description:  Wrapper.
**
******************************************************************************/
int main ( void )
{
	/* Call function to initialize the External Interrupts */	
	INTx_IO_Init();         

  /* Initialize the filter structure */	
  FIRDelayInit (&LowPassFilter);
  

  /* Initialize DCI and SI3000 Registers */
  libSi3000Init ();

  /* Start sampling speech data from SI3000 codec.Sampling of speech is stopped
     when number of samples played equals recordsize value specified in 
     CODECSETUP structure. */
  libSi3000StartSampling ( buffer1, buffer2,expandbuffer1,expandbuffer2,\
                               buffer11, buffer12 );

  libStartPlay ();                        //Enables the user to play speech.




    while (1)
    {
      /*If one Block of RAW speech data has been loaded in one of the ping pong
       buffers then start compressing that buffer*/
      if ( 1 == codecdata.fBlockdone )
      {

  		/* Filter application */
		if(UseFilter != 0)
		{
			int i;

			for(i = 0; i< FILTER_BLOCK_LENGTH; i++)
			{
				SigIn[i] = codecdata.sampleOpBuffer[i];
			}
  			FIR (FILTER_BLOCK_LENGTH, &FiltOut[0], &SigIn[0], &LowPassFilter);

			for(i = 0; i< FILTER_BLOCK_LENGTH; i++)
			{
				codecdata.sampleOpBuffer[i] = FiltOut[i];
			}
		}


        /* A Law Compress */
        ulaw_compress( slen ,codecdata.sampleOpBuffer , \
			          codecdata.sampleComprsIpBuffer );



        libBufManagerEncoder ();
        codecdata.fCompressdone = 0x01;
      }

      if( codecdata.fCompressdone == 0x01 )
      {
        /* U Law Expand */
        ulaw_expand( slen,codecdata.sampleComprsOpBuffer,
			        codecdata.sampleExpandIpBuffer );

        /* Rewind the sample buffer poniter */
        while ( codecdata.fBlockplayed != 0x00 );
        libBufManagerDecoder ();
	    codecdata.fCompressdone  = 0x00;
      }

      /* Check if the total number of blocks specified by #define RECORDSIZE
       in G711Lib_common.h have been played */
      if (  codecdata.recordSize == codecdata.loadblockCount )
         codecdata.loadblockCount = 0;   
     
    }

}

/*-----------------------------------------------------------------------------
               END OF FILE: G711SI3000LoopBack_Main.c 
-----------------------------------------------------------------------------*/
