/*********************************************************************
*                                                                    *
*                       Software License Agreement                   *
*                                                                    *
*   The software supplied herewith by Microchip Technology           *
*   Incorporated (the "Company") for its dsPIC controller            *
*   is intended and supplied to you, the Company's customer,         *
*   for use solely and exclusively on Microchip dsPIC                *
*   products. The software is owned by the Company and/or its        *
*   supplier, and is protected under applicable copyright laws. All  *
*   rights are reserved. Any use in violation of the foregoing       *
*   restrictions may subject the user to criminal sanctions under    *
*   applicable laws, as well as to civil liability for the breach of *
*   the terms and conditions of this license.                        *
*                                                                    *
*   THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION.  NO           *
*   WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING,    *
*   BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND    *
*   FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE     *
*   COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL,  *
*   INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.  *
*                                                                    *
*********************************************************************/

#include "p30fxxxx.h"
#include "G711Lib_common.h"
#include "G711Lib_Si3000.h"
#include "G711Lib_internal.h"

/******************************************************************************
**
**  FunctionName: libSi3000Init
**
**  Description:  This API initializes the DCI and SI 3000 codec registers
**                based on settings provided by the user through structure.It
**                accesses members of the dci structure for initializing the
**                above said registers.It has no outputs.
******************************************************************************/
void libSi3000Init ( void )
{
  unsigned int temp1,temp2;            //Temporary variables.
  if ( g711.dcimode == 1 )
  {
/*-----------------------------------------------------------------------------
      Cofiguration of DCI peripheral for setting up SI 3000 codec Registers.
-----------------------------------------------------------------------------*/
    DCICON1 = 0x0000;      			   //Ensure DCI module is placed in known
    DCICON2 = 0x0000;                  //state.
    DCICON3 = 0x0000;
    TSCON  = 0x0000;
    RSCON  = 0x0000;

    _TRISF6 = 0x01;       			   //Make codec reset switch port pin as 
                                       //inputfor now.Ensure port pins are 
                         			   //setcorrectly (Not really required 
                         		       //since DCI takes control of it's pins)

    TRISG = 0x5000;		 		       //CSDI pin configured for input
	    							   //COFS output to Si3000 in DCI Master 
			   			 			   //mode CSCK output to Si3000

    DCICON1 = 0x0040;	 			   //Set for multichannel frame sync mode
						 			   //Data tx/rx begins 1clock after Fs pulse
						 			   //Data changes on rising, sampled on falling
						 			   //Tristate output CSDO pin when not txing
						 			   //Frame sync generated by dsPIC DCI
						 			   //Clock is output from DCI
						 			   //Ensure module is disabled for now

    DCICON2 = 0x04EF;				   //Data word size is 16-bits (bits 3:0)
									   //Data frame is 8 words (bits 8:5)
									   //Set buffer length control (2 data words
									   //Will be buffered between interrupts)


    DCICON3 = g711.bcg1;			   //Initialize DCI bit clock generator


    RSCON = 0x0001;				       //Set receive slot 1 enable RSCON

    TSCON = 0x0001;				       //Set transmit slot 1 enable TSCON

    _DCIIF = 0x00;      			   //Ensure DCI interrupt flag is reset IFS2


    _DCIIE = 0x00;	    			   //ISR processing is disabled for now IEC2

/*-----------------------------------------------------------------------------
            Setup of SI 3000 codec through DCI peripheral on dsPIC30F device.
-----------------------------------------------------------------------------*/
    TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
                                       //requests a secondary frame

    TXBUF1 = (WRITE_PLL1_DIVIDE_N1|g711.pLL1divideN1); 
                          				
                                       //Load TXBUF1 with a value that will 
                            		   //initiate a WRITE to PLL Divide Control
                                       //Register 3 in the Si3000.



    _RF6 = 0x00;						   //Synchronize DCI with Si3000to
                                       //ensure first frame synce is
                                       //aligned with TXBUF0.
                                       //This is not really required
                                       //when the DCI is Master and
                                       //Codec is the Slave but is
                                       //required when the Codecis the
                                       //Master and DCI is the Slave.
    asm("nop");
    _TRISF6 = 0x00;		               //Reset codec
    asm("repeat  #40");                //Transmit a minimum pulse
    asm("nop");                        //width of 5uS

    _TRISF6 = 0x01;					   //Release codec

    _DCIEN = 0x1;			    	   //Enable DCI module
									   //This ensures the first primary
                                       //and secondary frame use data
                                       //loaded already into TXBUF0
                                       //and TXBUF1

    while ( ! _TMPTY );				   //Wait until TXBUF0 and TXBUF1
       							       //have been movedto their shadow
									   //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;  //Load TXBUF0 with a value that
       						           //requests a secondary frame

    TXBUF1 = (WRITE_PLL1_MULTIPLY_M1 | g711.pLL1multiplyM1);
                          							        
                                       //Load TXBUF1 with avalue that will ini-
                            		   //tiate aWRITE to PLL Multiply Register4
							            
    while( ! _TMPTY );                 //Wait until TXBUF0 and TXBUF1
                                	   //have been moved to their shadow
                                       //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;  //Load TXBUF0 with a value that
       							       //requests a secondary frame

    TXBUF1 = (WRITE_CONTROL_1  | g711.control1);
                                       //Load TXBUF1 with a value that
                            		   //will power up/down line driver
                               		   //and the speaker driver in Si3000
                             		   //Control Register 1

    while ( ! _TMPTY ); 				   //Wait until TXBUF0 and TXBUF1
                      				   //have been moved to their shadow
                                       //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;  //Load TXBUF0 with a value that
                           			   //requests a secondary frame

    TXBUF1 = (WRITE_CONTROL_2 | g711.control2); //Load TXBUF1 with a value that
                          				//will initialize Control2
                                        //Register 2.
    while ( ! _TMPTY );

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;	//Load TXBUF0 with a value that
                           				//requests a secondary frame

    TXBUF1 = (WRITE_RX_GAIN_CONTROL_1 | g711.rxgaincontroL1);//Load TXBUF1 with
                          				//a value that will initialize
                                        //RX Gain Control1 Register.


    while ( ! _TMPTY);					//Wait until TXBUF0 and TXBUF1
                         				//have been moved to their shadow
                                        //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;    //Load TXBUF0 with a value that
                                        //requests a secondary frame

    TXBUF1 = (WRITE_ADC_VOLUME_CONTROL  | g711.adcvolumecontrol);//Load TXBUF1
                            			//with a value that will
                                        //initialize ADC volume contol
                                        //Register6.
    while ( ! _TMPTY );					//Wait until TXBUF0 and TXBUF1
                         				//have been moved to their shadow
                                        //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
                     					//requests a secondary frame

    TXBUF1 = (WRITE_DAC_VOLUME_CONTROL |g711.dacvolumecontrol);//Load TXBUF1
                           				// with a value that that will
                                        //initialize DAC Volume Control 
                                        // Register7   

    while ( ! _TMPTY );					//Wait until TXBUF0 and TXBUF1
                          			    //have been moved to their shadow
                                        //registers for transmission

    TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
                      					//requests a secondary frame
    TXBUF1 = (WRITE_ANALOG_ATTENUATION | g711.analogattenuation);//Load TXBUF1
                           				//with a value that that
                                        //will initialize Analog
                                        //attenuation Control Register9.
    while(!_TMPTY);						//Wait until TXBUF0 and TXBUF1
                          				//have been moved to their shadow
                                        //registers for transmission

/*-----------------------------------------------------------------------------
Note that the TMPTY bit only gives a status of whether or not TXBUF registers
are ready to accept data. This does not tell you if the transmissions themselves
were completed.So, we need to test the SLOT<3:0> bits in the DCISTAT register.
While testing we will ensure that TXBUF0 and TXBUF1 shadow registers have been
fully transmitted on TimeSlot0 two times each. This will guarantee that
initialization of the codec is complete
-----------------------------------------------------------------------------*/
    temp1 = 0x0f00;						//Set up a mask for the SLOT bits
    do
    {									//Read the DCISTAT register
      temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
      temp2 &= temp1;
	} while ( temp2 ); 					//Wait until SLOT bits reach 0b0000
 								        //Did TXBUF0 get transmitted?

	do
    {									//Read the DCISTAT register
      temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
      temp2 &= temp1;
	} while ( ! temp2 );					//Wait until SLOT bits reach non-zero
          								//Did TXBUF1 get transmitted?

	do
    {									//Read the DCISTAT register
      temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
      temp2 &= temp1;
	} while ( temp2 ); 					//Wait until SLOT bits reach 0b0000
          								//Did TXBUF0 get transmitted?

	do
    {									//Read the DCISTAT register
      temp2= DCISTAT;					//Logical-AND to extract SLOT bits
      temp2 &= temp1;
	} while ( ! temp2 );				//Wait until SLOT bits reach 0b0000
        								//Did TXBUF0 get transmitted?

	while ( ! _ROV );					//Testing overflow bit

    temp1 = RXBUF0;						//Dummy reads to clear
    temp1 = RXBUF1;						//overflow conditions

	_DCIEN = 0x00;						//Disable the DCI module

/*-----------------------------------------------------------------------------
Now re-initialize some aspects of the DCI communication for normal interrupt
driven operation based on user settings done in dci and codecsetup structures.
-----------------------------------------------------------------------------*/

    DCICON1 = DCICON1VAL;    		    //Set mode.
										//Data txmit/rx setup.
										//Data change setup.
              							//Tristate/set low output CSDO pin when
                                        //not txing.
										//Frame sync generated by dsPIC DCI/SI 3000.
										//Clock is output from DCI/SI 3000.
										//Ensure module is disabled for now
    DCICON2 = DCICON2VAL;			    //Set Data word size .(bits 3:0)
										//Set data frame size.(bits 8:5)
    DCICON3 = g711.bcg1;                //Set DCI bit clock generator.
    RSCON = RSCONVAL;                    //Set receive slot enable bits.
    TSCON = TSCONVAL;                    //Set transmit slot enable bits.
    codecdata.countFill = 0x0000;        //Clear counter that keeps track of data
                                        //filled into ping pong buffers.
    codecdata.countLoad = 0x0000;        //Clear counter that keeps track of data
                                        //retrieved from ping pong buffers.

    codecdata.fBlockdone = 0x00;			//Clear flag that indicates whether or
 										//not one sample ping pong buffer
    _DCIEN = 0x01;                      //has been filled.
 }


  else
  {

/*-----------------------------------------------------------------------------
      Cofiguration of DCI peripheral for setting up SI 3000 codec Registers.
-----------------------------------------------------------------------------*/
    DCICON1 = 0x0000;      		       //Ensure DCI module is placed in known
    DCICON2 = 0x0000;                   //state.
    DCICON3 = 0x0000;
    TSCON   = 0x0000;
    RSCON   = 0x0000;
    _TRISF6 = 0x01;       			   //Make codec reset switch port pin as 
                                       //inputfor now.Ensure port pins are 
                         			   //setcorrectly (Not really required 
                         		       //since DCI takes control of it's pins)

    //TRISG = 0x5000;		 		   //CSDI pin configured for input
    TRISG = 0xD000;	    			   //COFS output to Si3000 in DCI Master 
			   			 			   //mode CSCK output to Si3000

    DCICON1 = 0x0040;	 			   //Set for multichannel frame sync mode
						 			   //Data tx/rx begins 1clock after Fs pulse
						 			   //Data changes on rising, sampled on falling
						 			   //Tristate output CSDO pin when not txing
						 			   //Frame sync generated by dsPIC DCI
						 			   //Clock is output from DCI
						 			   //Ensure module is disabled for now

    DCICON2 = 0x04EF;				   //Data word size is 16-bits (bits 3:0)
									   //Data frame is 8 words (bits 8:5)
									   //Set buffer length control (2 data words
									   //Will be buffered between interrupts)


    DCICON3 = 0x0000;       			   //Initialize DCI bit clock generator

   
    _COFSD = 0x01;
    _CSCKD = 0x01;

    RSCON = 0x0001;				       //Set receive slot 1 enable RSCON

    TSCON = 0x0001;				       //Set transmit slot 1 enable TSCON

    _DCIIF = 0x00;      			       //Ensure DCI interrupt flag is reset IFS2


    _DCIIE = 0x00;	    			   //ISR processing is disabled for now IEC2





    _LATF1 = 0x01;
    _LATF2 = 0x01;
    _TRISF1 = 0x00;
    _TRISF2 = 0x00;




/*-----------------------------------------------------------------------------
            Setup of SI 3000 codec through DCI peripheral on dsPIC30F device.
-----------------------------------------------------------------------------*/
    TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
                                       //requests a secondary frame

    TXBUF1 = (WRITE_PLL1_DIVIDE_N1|g711.pLL1divideN1); 
                          				
                                       //Load TXBUF1 with a value that will 
                            		   //initiate a WRITE to PLL Divide Control
                                       //Register 3 in the Si3000.


    _LATF1 = 0x00;
    _LATF2 = 0x00;
    _TRISF1 = 0x00;
    _TRISF2 = 0x00;
    asm("REPEAT #200");
    asm("nop");

   _LATF1 = 0x01;
   _LATF2 = 0x01;

   _RF6 = 0x00;						   //Synchronize DCI with Si3000to
                                       //ensure first frame synce is
                                       //aligned with TXBUF0.
                                       //This is not really required
                                       //when the DCI is Master and
                                       //Codec is the Slave but is
                                       //required when the Codecis the
                                       //Master and DCI is the Slave.
   asm("nop");
   _TRISF6 = 0x00;		               //Reset codec
   asm("repeat  #40");                 //Transmit a minimum pulse
   asm("nop");                         //width of 5uS

   _TRISF6 = 0x01;					   //Release codec

   _DCIEN = 0x1;			    	   //Enable DCI module
									   //This ensures the first primary
                                       //and secondary frame use data
                                       //loaded already into TXBUF0
                                       //and TXBUF1

//-------------------------------------------------------



   while ( ! _TMPTY );				   //Wait until TXBUF0 and TXBUF1
       							       //have been movedto their shadow
									   //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
       						           //requests a secondary frame

   TXBUF1 = (WRITE_PLL1_MULTIPLY_M1 | g711.pLL1multiplyM1);
                          							        
                                       //Load TXBUF1 with avalue that will ini-
                            		   //tiate aWRITE to PLL Multiply Register4
							            
   while( ! _TMPTY );                  //Wait until TXBUF0 and TXBUF1
                                	   //have been moved to their shadow
                                       //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
       							       //requests a secondary frame

   TXBUF1 = (WRITE_CONTROL_1  | g711.control1);
                                       //Load TXBUF1 with a value that
                            		   //will power up/down line driver
                               		   //and the speaker driver in Si3000
                             		   //Control Register 1

   while ( ! _TMPTY ); 				   //Wait until TXBUF0 and TXBUF1
                      				   //have been moved to their shadow
                                       //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;   //Load TXBUF0 with a value that
                           			   //requests a secondary frame

   TXBUF1 = (WRITE_CONTROL_2 | g711.control2); //Load TXBUF1 with a value that
                          				//will initialize Control2
                                        //Register 2.
   while ( ! _TMPTY );

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;	//Load TXBUF0 with a value that
                           				//requests a secondary frame

   TXBUF1 = (WRITE_RX_GAIN_CONTROL_1 | g711.rxgaincontroL1);//Load TXBUF1 with
                          				//a value that will initialize
                                        //RX Gain Control1 Register.


   while ( ! _TMPTY);					//Wait until TXBUF0 and TXBUF1
                         				//have been moved to their shadow
                                        //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;    //Load TXBUF0 with a value that
                                        //requests a secondary frame

   TXBUF1 = (WRITE_ADC_VOLUME_CONTROL  | g711.adcvolumecontrol);//Load TXBUF1
                            			//with a value that will
                                        //initialize ADC volume contol
                                        //Register6.
   while ( ! _TMPTY );					//Wait until TXBUF0 and TXBUF1
                         				//have been moved to their shadow
                                        //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;    //Load TXBUF0 with a value that
                     					//requests a secondary frame

   TXBUF1 = (WRITE_DAC_VOLUME_CONTROL |g711.dacvolumecontrol);//Load TXBUF1
                           				// with a value that that will
                                        //initialize DAC Volume Control 
                                        // Register7   

   while ( ! _TMPTY );					//Wait until TXBUF0 and TXBUF1
                          			    //have been moved to their shadow
                                        //registers for transmission

   TXBUF0 = PLUS_ONE_WITH_SECONDARY;    //Load TXBUF0 with a value that
                      					//requests a secondary frame
   TXBUF1 = (WRITE_ANALOG_ATTENUATION | g711.analogattenuation);//Load TXBUF1
                           				//with a value that that
                                        //will initialize Analog
                                        //attenuation Control Register9.
   while(!_TMPTY);						//Wait until TXBUF0 and TXBUF1
                          				//have been moved to their shadow
                                        //registers for transmission

/*-----------------------------------------------------------------------------
Note that the TMPTY bit only gives a status of whether or not TXBUF registers
are ready to accept data. This does not tell you if the transmissions themselves
were completed.So, we need to test the SLOT<3:0> bits in the DCISTAT register.
While testing we will ensure that TXBUF0 and TXBUF1 shadow registers have been
fully transmitted on TimeSlot0 two times each. This will guarantee that
initialization of the codec is complete
-----------------------------------------------------------------------------*/
   temp1 = 0x0f00;						//Set up a mask for the SLOT bits
   do
   {									//Read the DCISTAT register
     temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
     temp2 &= temp1;
   } while ( temp2 ); 					//Wait until SLOT bits reach 0b0000
 								        //Did TXBUF0 get transmitted?

	 do
   {									//Read the DCISTAT register
     temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
     temp2 &= temp1;
   } while ( ! temp2 );					//Wait until SLOT bits reach non-zero
          								//Did TXBUF1 get transmitted?

	 do
   {									//Read the DCISTAT register
     temp2 = DCISTAT;					//Logical-AND to extract SLOT bits
     temp2 &= temp1;
   } while ( temp2 ); 					//Wait until SLOT bits reach 0b0000
          								//Did TXBUF0 get transmitted?

	 do
   {									//Read the DCISTAT register
     temp2= DCISTAT;					//Logical-AND to extract SLOT bits
     temp2 &= temp1;
   } while ( ! temp2 );					//Wait until SLOT bits reach 0b0000
        								//Did TXBUF0 get transmitted?

	 while ( ! _ROV );					//Testing overflow bit

   temp1 = RXBUF0;						//Dummy reads to clear
   temp1 = RXBUF1;						//overflow conditions

	 _DCIEN = 0x00;						//Disable the DCI module

/*-----------------------------------------------------------------------------
Now re-initialize some aspects of the DCI communication for normal interrupt
driven operation based on user settings done in dci and codecsetup structures.
-----------------------------------------------------------------------------*/

   DCICON1 = DCICON1VAL;    		    //Set mode.
										//Data txmit/rx setup.
										//Data change setup.
              							//Tristate/set low output CSDO pin when
                                        //not txing.
										//Frame sync generated by dsPIC DCI/SI 3000.
										//Clock is output from DCI/SI 3000.
										//Ensure module is disabled for now
   DCICON2 = DCICON2VAL;			    //Set Data word size .(bits 3:0)
										//Set data frame size.(bits 8:5)
   // DCICON3 = g711.bcg1;                //Set DCI bit clock generator.
   DCICON3 = 0x0000;
   _COFSD = 0x01;
   _CSCKD = 0x01; 
   RSCON = RSCONVAL;                    //Set receive slot enable bits.
   TSCON = TSCONVAL;                    //Set transmit slot enable bits.
   codecdata.countFill = 0x0000;        //Clear counter that keeps track of data
                                        //filled into ping pong buffers.
   codecdata.countLoad = 0x0000;        //Clear counter that keeps track of data
                                        //retrieved from ping pong buffers.

   codecdata.fBlockdone = 0x00;			//Clear flag that indicates whether or
 										//not one sample ping pong buffer
                                        //has been filled.
   _DCIEN = 0x01;
   

	  
   temp1 = 0x0100;
   do
   {
     temp2 = 0xFFFF;

     while(--temp2);
 
   } while(--temp1);
  }
}
/******************************************************************************
**
**  Function Name: libSi3000DCIFill ( )
**
**  Description:   This API is called in the DCI ISR for moving data from the
**                 DCI buffer registers RXBUFx's to the user defined ping pong
**                 buffers.It accesses members of the codecsetup structure. It
**                 has no outputs.The user should take care to set the DCI
**                 buffer length such that the number of samples per frame
**                 should be divisible by it.
**
******************************************************************************/
void libSi3000DCIFill ( void )
{
  short *temp1;                          //Temporary variable.

  *( codecdata.sampleIpBuffer ) = RXBUF0;//DCI buffer length equal to 4.
  ( codecdata.sampleIpBuffer )++;
  *( codecdata.sampleIpBuffer ) = RXBUF1;
  ( codecdata.sampleIpBuffer )++;
  *( codecdata.sampleIpBuffer ) = RXBUF2;
  ( codecdata.sampleIpBuffer )++;
  *( codecdata.sampleIpBuffer ) = RXBUF3;
  ( codecdata.sampleIpBuffer )++;
  codecdata.countFill += BUFFERLENGTH;

  TXBUF0 = 0x0000;                     //Send digital silence to SI3000 codec.
  TXBUF1 = 0x0000;
  TXBUF2 = 0x0000;
  TXBUF3 = 0x0000;

  /* Check if the buffer has been filled to its capacity. */
  if ( codecdata.countFill == NUMOFSAMPLESPERBLOCK )
  {
    codecdata.countFill = 0x0000;     //Clear data fill counter

    /* Rewind to the begining of  the sample buffer. */
	(codecdata.sampleIpBuffer) -= ( NUMOFSAMPLESPERBLOCK );

    /* Exchange the addresses of sample buffer pointers in to the pointer
       to sample buffers to manage the user defined ping pong buffers */
	temp1                    = codecdata.sampleIpBuffer;
	codecdata.sampleIpBuffer = codecdata.sampleOpBuffer;
    codecdata.sampleOpBuffer = temp1;

    /* Set the flag to indicate that one buffer has filled to its capacity and
      is ready for use */
    codecdata.fBlockdone = 0x01;
    (codecdata.blockCount)++;
  }
}
/******************************************************************************
**
**  Function Name: libSi3000LoadDCI ( ) 
**
**  Description:   This API is called in the DCI ISR for moving data from the
**                 user defined ping pong buffers to the DCI buffer registers
**                 TXBUFx's.It accesses members of codecsetup structure. It has
**                 no outputs.The user should take care to set the DCI buffer
**                 length such that the number of samples per Block should be
**                 divisible by it.
**
******************************************************************************/
void libSi3000LoadDCI ( void )
{

  /*The user should have given command for playing and one of the ping pong
    buffer should have been filled to its capacity.*/
  if ( codecdata.fStartPlay == 0x01)
  {
    {        
      /* DCI buffer length equal to 4. */ 
      TXBUF0 = ( *codecdata.sampleExpandOpBuffer  );/* Shifting for Alaw */
      (  codecdata.sampleExpandOpBuffer )++;
      TXBUF1 = ( *codecdata.sampleExpandOpBuffer  );
      (  codecdata.sampleExpandOpBuffer  )++;
      TXBUF2 = ( *codecdata.sampleExpandOpBuffer  );
      (  codecdata.sampleExpandOpBuffer  )++;
      TXBUF3 = ( *codecdata.sampleExpandOpBuffer  );
      (  codecdata.sampleExpandOpBuffer )++;
      codecdata.countLoad  += BUFFERLENGTH;
    }
    if ( codecdata.countLoad  == NUMOFSAMPLESPERBLOCK )
	{
     
      /*After one of the ping pong sample buffer's has been played clear the
        load counter and increment blockCount.*/
      codecdata.countLoad  = 0x0000;
      ( codecdata.loadblockCount )++;

      /*Rewind the sample buffer pointer to the begining of the sample buffer*/
      codecdata.sampleExpandOpBuffer -= (NUMOFSAMPLESPERBLOCK );

      /* Clear the flag to indicate that the sample buffer had been played */
      codecdata.fBlockplayed = 0x00;
    }
  }
}
/******************************************************************************
**
**  FunctionName: libSi3000StartSampling ( )
**
**  Description:  This API enables the user to start sampling speech data from
**                SI3000 codec.This API has six inputs which have to be passed
**                to it as parameters, it also accesses the codecsetup
**                structure members.Both the parameters are pointers to user
**                defined sample buffers.These buffers will be handled as ping
**                pong buffers.i.e.when data is being filled in one buffer the
**                other buffer will be used to retrieve already stored data and
**                vice versa when the other buffer is being filled with data.
**
******************************************************************************/
void libSi3000StartSampling (
 short *buf1,
 short *buf2,
 short *expandbuf1,
 short *expandbuf2,
 short *buffer1,
 short *buffer2 )
{
  /* Initialize the sample buffer 1 pointer to one of the user defined ping
     pong buffers */
  codecdata.sampleIpBuffer = buf1;

  /* Initialize the sample buffer 2 pointer to the other user defined ping
     pong sample buffer */
  codecdata.sampleOpBuffer = buf2;

  /* Initialize the sample buffer 1 pointer to one of the user defined ping
     pong buffers */
  codecdata.sampleExpandIpBuffer = expandbuf1;

  /* Initialize the sample buffer 2 pointer to the other user defined ping
     pong sample buffer */
  codecdata.sampleExpandOpBuffer = expandbuf2;

  /* Initialize the sample buffer 11 pointer to one of the user defined ping
     pong buffers */
  codecdata.sampleComprsIpBuffer = buffer1;

  /* Initialize the sample buffer 12 pointer to one of the user defined ping
     pong buffers */
  codecdata.sampleComprsOpBuffer = buffer2;

  codecdata.blockCount = 0x0000;       //Clear the flag for playing new record.

  _DCIIF = 0x00;				       //Clear the DCI interrupt flag
  _DCIIP =  g711.dciintpri;            //Set Interrupt Priority of DCI.

  /* Send digital silence to the output And start sampling speech data from
     SI 3000 */
  TXBUF0 = 0x0000;
  TXBUF1 = 0x0000;
  TXBUF2 = 0x0000;
  TXBUF3 = 0x0000;

  _DCIIF = 0x00;                       //Clear DCI interrupt flag .
  _DCIIE = 0x01;					   //Enable DCI interrupts
  _DCIEN = 0x01;					   //Enable the DCI peripheral. 							             
}
/******************************************************************************
**
**  Function Name: libSi3000StopSampling ( )
**
**  Description:  This API enables the user to stop sampling speech data from 
**                SI3000 codec.It has no inputs and outputs.It accesses the 
**                DCI Registers directly.
******************************************************************************/ 
void libSi3000StopSampling ( void ) 
{
  _DCIIF = 0x00;  							//Clear the DCI interrupt flag   
  _DCIIE = 0x00;							//Disable DCI interrupts
  _DCIEN = 0x00;							//Disable DCI peripheral.
}
/******************************************************************************
**
**  FunctionName: libStartPlay ( )
**
**  Description:  This API enables the user to play speech.This function has no
**                inputs.It sets the fStartPlay flag in the codecsetup struc-
**                ture.This flag is checked in the libFullDuplexSi3000LoadDCI()
**               functions.If it is set then speech is played through the 
**                Output Compare Module and SI 3000 codec respectively.
******************************************************************************/
void libStartPlay( void ) 
{
  codecdata.fStartPlay = 0x01;
}
/******************************************************************************
**
**  FunctionName: libBufManagerDecoder
**
**  Description:  This API does ping pong buffer management after decoding is
**                done.
******************************************************************************/
void libBufManagerDecoder ( void )
{
  short *temp;                                  /* Temporary variable. */
  /* Exchange the addresses of sample buffer pointers in to the pointer
     to sample buffers to manage the user defined ping pong buffers */

  temp                           = codecdata.sampleExpandIpBuffer;
  codecdata.sampleExpandIpBuffer = codecdata.sampleExpandOpBuffer;
  codecdata.sampleExpandOpBuffer = temp;

  /* Clear the flag to indicate that the sample buffer has been transmitted. */
  codecdata.fBlockdone  = 0x00;

  /* Set the flag which is to indicate decode is done */
  codecdata.fBlockplayed = 0x01;
}
/******************************************************************************
**
**  FunctionName: libBufManagerEncoder
**
**  Description:  This API does ping pong buffer management after encoding is
**                done.
******************************************************************************/
void libBufManagerEncoder ( void )
{
  short* temp;

  /* Exchange the addresses of sample Encd buffer pointers in to the pointer
      to sample Encd buffers to manage the user defined ping pong buffers */

  temp                        	= codecdata.sampleComprsIpBuffer;
  codecdata.sampleComprsIpBuffer  = codecdata.sampleComprsOpBuffer;
  codecdata.sampleComprsOpBuffer  = temp;
  codecdata.fBlockdone          = 0x00;
  codecdata.fEncodedone         = 0x01;
}
/******************************************************************************
**
**  FunctionName: _DCIInterrupt
**
**  Description:  This ISR for DCI interrupt handles data reads from DCI buffer 
**                registers RXBUFx,gives for compression and loads the expanded
**                samples to DCI buffer registers TXBUFx to/from the ping pong
**                buffers created by the user.
**                             
******************************************************************************/
void __attribute__((__interrupt__))  _DCIInterrupt ( void )
{
   
  /* This function fills data into ping pong buffer */
  libSi3000DCIFill ( );

  if( codecdata.fBlockplayed == 0x01 )
  { 
    /* This function loads data from ping pong buffers to DCI TXBUFs registers */
    libSi3000LoadDCI ( );
  }

  _DCIIF = 0x00;
}
